@@ -45,15 +45,29 @@ Variable <- R6::R6Class(
                           type = "free",
                           domain = NULL, records = NULL,
                           domainForwarding = FALSE,
-                          description="") {
+                          description="", ...) {
 
-      self$type = type
+      args = list(...)
+      from_gdx = args[["from_gdx"]]
+      if (is.null(from_gdx)) from_gdx=FALSE
+
+      if (from_gdx) {
+        private$.type = type
+      }
+      else {
+        self$type = type
+      }
 
       super$initialize(container, name,
-                      domain, description, domainForwarding)
+                      domain, description, domainForwarding, from_gdx=from_gdx)
 
       if (!is.null(records)) {
-        self$setRecords(records)
+        if (from_gdx) {
+          private$.records = records
+        }
+        else {
+          self$setRecords(records)
+        }
       }
     },
 
@@ -126,10 +140,10 @@ Variable <- R6::R6Class(
         if (self$dimension >= 2) {
           for (i in names(records)) {
             recs = records[[i]]
-            if (!all(dim(recs) == self$shape())) {
+            if (!all(dim(recs) == self$shape)) {
               stop(paste0("User passed array/matrix with shape ", 
               toString(dim(recs)), " but anticipated shape was ", 
-              toString(self$shape()), " based on domain set information ",
+              toString(self$shape), " based on domain set information ",
               "-- must reconcile before ",
               "array-to-records conversion is possible.\n"))
             }
